﻿using System.Web.Routing;
using Nop.Core.Plugins;
using Nop.Plugin.Misc.DynamicsCrm.Services;
using Nop.Services.Common;

namespace Nop.Plugin.Misc.DynamicsCrm
{
    public class DynamicsCrmPlugin : BasePlugin, IMiscPlugin
    {
        private readonly DynamicsCrmInstallationService _dynamicsCrmInstallationService;
        private readonly DynamicsCrmSettings _dynamicsCrmSettings;

        public DynamicsCrmPlugin(DynamicsCrmInstallationService dynamicsCrmInstallationService,
            DynamicsCrmSettings dynamicsCrmSettings)
        {
            this._dynamicsCrmInstallationService = dynamicsCrmInstallationService;
            this._dynamicsCrmSettings = dynamicsCrmSettings;
        }

        /// <summary>
        /// Is plugin configured?
        /// </summary>
        /// <returns></returns>
        public virtual bool IsConfigured()
        {
            return !string.IsNullOrEmpty(_dynamicsCrmSettings.UserName);
        }

        /// <summary>
        /// Install plugin
        /// </summary>
        public override void Install()
        {
            _dynamicsCrmInstallationService.Install(this);
            base.Install();
        }

        /// <summary>
        /// Uninstall plugin
        /// </summary>
        public override void Uninstall()
        {
            _dynamicsCrmInstallationService.Uninstall(this);
            base.Uninstall();
        }

        /// <summary>
        /// Gets a route for plugin configuration
        /// </summary>
        /// <param name="actionName">Action name</param>
        /// <param name="controllerName">Controller name</param>
        /// <param name="routeValues">Route values</param>
        public void GetConfigurationRoute(out string actionName, out string controllerName, out RouteValueDictionary routeValues)
        {
            actionName = "Index";
            controllerName = "Settings";
            routeValues = new RouteValueDictionary { { "Namespaces", "Nop.Plugin.Misc.DynamicsCrm.Controllers" }, { "area", null } };
        }
    }
}